package com.fairphone.moduledetect;

import android.content.Context;
import android.content.res.Resources;
import android.util.Log;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * Wrapper for finding currently installed front and main camera sensors
 */

public class CameraDetect {

    private static final String TAG = "CameraDetect";

    private static final String VIDEO4LINUX_SYSFS_PATH =
            "/sys/devices/fd8c0000.qcom,msm-cam/video4linux";

    private String mainSensorName = "";
    private String frontSensorName = "";

    public CameraDetect(Context context) {
        Resources res = context.getResources();

        setMainSensorName(runCameraDetectCommand(res.getStringArray(R.array.sensor_names_main)));
        setFrontSensorName(runCameraDetectCommand(res.getStringArray(R.array.sensor_names_front)));
    }

    public String getMainSensorName() {
        return mainSensorName;
    }

    public String getFrontSensorName() {
        return frontSensorName;
    }

    private void setMainSensorName(String mainSensorName) {
        this.mainSensorName = mainSensorName;
    }

    private void setFrontSensorName(String frontSensorName) {
        this.frontSensorName = frontSensorName;
    }

    /**
     * Find which sensor name of {@code sensorNames} is installed
     *
     * @param sensorNames List of sensor names to match against
     * @return First sensor name that is matched. Empty String otherwise.
     */
    private static String runCameraDetectCommand(final String[] sensorNames) {
        final List<String> sensorList = Arrays.asList(sensorNames);
        class FindSensor implements FileVisitor<Path> {
            public String foundSensor;
            @Override
            public FileVisitResult preVisitDirectory(Path path, BasicFileAttributes basicFileAttributes) {return FileVisitResult.CONTINUE; }
            @Override
            public FileVisitResult visitFile(Path path, BasicFileAttributes basicFileAttributes) {
                if (!path.getFileName().toString().equals("name")) {
                    return FileVisitResult.CONTINUE;
                }
                try {
                    Optional<String> deviceName = Files.lines(path, StandardCharsets.US_ASCII).findFirst();
                    if (deviceName.isPresent() && sensorList.contains(deviceName.get())) {
                        foundSensor = deviceName.get();
                        return FileVisitResult.TERMINATE;
                    }
                } catch (IOException e) {
                    Log.e(TAG, "Failure reading sysfs entry " + path + ": " + e);
                }
                return FileVisitResult.CONTINUE;
            }
            @Override
            public FileVisitResult visitFileFailed(Path path, IOException e) { return FileVisitResult.CONTINUE; }
            @Override
            public FileVisitResult postVisitDirectory(Path path, IOException e) { return FileVisitResult.CONTINUE; }
        }

        FindSensor findSensor = new FindSensor();
        try {
            Files.walkFileTree(Paths.get(VIDEO4LINUX_SYSFS_PATH), findSensor);
            if (findSensor.foundSensor != null) {
                return findSensor.foundSensor;
            }
        } catch (IOException e) {
            Log.e(TAG, "Failure while walking sysfs: " + e);
        }

        return "";
    }
}
